<?php
/**
 * Public facing specific functionality.
 *
 * @since         4.0.0
 * @package       WP_NoExternalLinks
 * @subpackage    WP_NoExternalLinks/Public
 * @author        SteamerDevelopment
 */

# PHP WP No External Links script modification by W. Al Maawali  
# (c) 2017 Founder of Eagle Eye Digital Solutions
# http://www.digi77.com
# http://www.om77.net
# Script starts here:
# Tutorial link: https://www.digi77.com/877extracting-ssl-fingerprints/
# To see my changes just search for the word Warith
# Original Plugin version 4.0.2 by Jehy can be found here https://wordpress.org/plugins/wp-noexternallinks/
 
 
class WP_NoExternalLinks_Public {

	/**
	 * The ID of this plugin.
	 *
	 * @since    4.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    4.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

    /**
     * The options of this plugin.
     *
     * @since    4.0.0
     * @access   private
     * @var      object    $options    The current options of this plugin.
     */
    private $options;

    /**
     * The site url.
     *
     * @since    4.0.0
     * @access   private
     * @var      string    $site    The site url.
     */
    private $site;

    /**
     * The exclusion list of this plugin.
     *
     * @since    4.0.0
     * @access   private
     * @var      string    $version    The exclusion list of this plugin.
     */
    private $exclusion_list;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    4.0.0
	 * @param    string    $plugin_name    The name of the plugin.
	 * @param    string    $version        The version of this plugin.
     * @param    object    $options        The current options of this plugin.
	 */
	public function __construct( $plugin_name, $version, $options ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;
        $this->options = $options;

        $this->site = get_option('home') ? get_option('home') : get_option('siteurl');

        $this->exclusion_list = $this->get_exclusion_list();

	}

    /**
     * Start full page filtering.
     *
     * @since    4.0.0
     */
    public function fullmask_begin() {

        if ( defined( 'DOING_CRON' ) ) {
            // do not try to use output buffering on cron
            return;
        }

        ob_start( array( $this, 'fullmask_end') );

        $this->debug_info( "Starting full mask." );

    }

    /**
     * Filter full page content.
     *
     * @since     4.0.0
     * @param     string    $text
     * @return    string
     */
    public function fullmask_end( $text ) {

        global $post;

        // do not try to use output buffering on cron
        if ( defined( 'DOING_CRON' ) ) {
            return '';
        }

        $r = '';
        $r .= $this->debug_info( "Full mask finished. Applying filter", 1 );

        if ( $text ) {
            $r .= $this->debug_info( "Processing text (htmlspecialchars on it to stay like comment): \n" . htmlspecialchars($text), 1 );

            if ( is_object( $post ) && ( get_post_meta( $post->ID, 'mask_links', true ) === 'disabled' ) ) {
                $r .= $text;
            } elseif ( function_exists( 'is_feed' ) && is_feed() ) {
                $r .= $text;
            } else {
                $r .= $this->filter( $text );
            }
        }

        $r .= $this->debug_info( "Full mask output finished", 1 );

        return $r;

    }

    /**
     * Check if post/page should be masked.
     *
     * @since     4.0.0
     * @param     string    $content
     * @return    string    mixed
     */
    public function check_post( $content ) {

        global $post;

        $this->debug_info( 'Checking post for meta.' );

        if ( 'disabled' === get_post_meta( $post->ID, 'mask_links', true ) ) {
            $this->debug_info( 'Meta nomask. No masking will be applied' );

            return $content;
        } else {
            $this->debug_info( 'Filter will be applied' );

            return $this->filter( $content );
        }

    }

    /**
     * Filters content to find links.
     *
     * @since     4.0.0
     * @param     string    $content
     * @return    mixed
     */
    public function filter( $content ) {

        if ( function_exists('is_admin') && is_admin() ) {
            return $content;
        }

        $this->debug_info( "Processing text: \n" . str_replace( '-->', '--&gt;', $content ) );

        if ( function_exists('is_feed') && is_feed() && ! $this->options->mask_rss && ! $this->options->mask_rss_comments ) {
            $this->debug_info( 'It is feed, no processing' );

            return $content;
        }

        $pattern = '/<a (.*?)href=[\"\'](.*?)[\"\'](.*?)>(.*?)<\/a>/si';

        $content = preg_replace_callback( $pattern, array( $this, 'parser' ), $content, -1, $count );

        $this->debug_info( $count . " replacements done.\nFilter returned: \n" . str_replace( '-->', '--&gt;', $content ) );

        return $content;

    }

    /**
     * Performs link masking.
     *
     * @since     4.0.0
     * @param     array     $matches
     * @return    string
     */
    public function parser( $matches ) {

        global $wp_rewrite;

        $url = $matches[2];

        $this->debug_info( 'Parser called. Parsing argument {' . var_export( $matches, 1 ) . "}\nAgainst link {" . $url . "}\n " );

        $r = $this->check_exclusions( $matches );

        if ( false !== $r ) {
            return $r;
        }

        if ( ! $wp_rewrite->using_permalinks() ) {
            $sep = '?' . $this->options->separator . '=';
        } else {
            $sep = '/' . $this->options->separator . '/';
        }

        $blank = '';
        if ( $this->options->target_blank ) {
            $blank = ' target="_blank"';
        }

        $nofollow = '';
        if ( $this->options->nofollow ) {
            $nofollow = ' rel="nofollow"';
        }

        if ( 'no' !== $this->options->masking_type ) {
            $url = $this->encode_link( $url );

            if ( ! $wp_rewrite->using_permalinks() ) {
                $url = urlencode( $url );
            }

            if ( '/' == $sep[0] ) {
                $sep = substr( $sep, 1 );
            }

            $tmp = $this->site;

            // add "/" to site url- some servers don't work with urls like xxx.ru?goto, but with xxx.ru/?goto
            if ( substr( $this->site, 0, -1 ) !== '/' ) {
                $tmp .= '/';
            }

            $url = $tmp . $sep . $url;
        }

        if ( $this->options->remove_all_links ) {
            return '<span class="waslinkname">' . $matches[4] . '</span>';
        }

        if ( $this->options->links_to_text ) {
            return '<span class="waslinkname">' . $matches[4] . '</span> ^(<span class="waslinkurl">' . $url . ')</span>';
        }

        $link = '<a' . $blank . $nofollow . ' href="' . $url . '"' . $matches[1] . $matches[3] . '>' . $matches[4] . '</a>';

        if ( $this->options->noindex_tag ) {
            $link = '<noindex>' . $link . '</noindex>';
        }

        if ( $this->options->noindex_comment ) {
            $link = '<!--noindex-->' . $link . '<!--/noindex-->';
        }

        return $link;
    }

    /**
     * Checks possible exclusions.
     *
     * @since     4.0.0
     * @param     array    $matches
     * @return    mixed
     */
    public function check_exclusions( $matches ) {

        if ( $r = $this->check_follow( $matches ) ) {
            return $r;
        }

        if ( $r = $this->check_excl_list( $matches ) ) {
            return $r;
        }

        return false;

    }

    /**
     * Checks if follow links should be masked.
     *
     * @since     4.0.0
     * @param     array    $matches
     * @return    mixed
     */
    public function check_follow( $matches ) {

        // support of "meta=follow" option for admins. disabled by default to minify processing.
        if ( ! $this->options->skip_follow ) {
            return false;
        }

        // it is either page or post
        $id = array( get_comment_ID(), get_the_ID() );

        if ( $id[0] ) {
            $this->debug_info( 'It is a comment. id ' . $id[0] );
        } elseif ( $id[1] ) {
            $this->debug_info( 'It is a page. id ' . $id[1] );
        }

        $author = false;

        if ( $id[0] ) {
            $author = get_comment_author( $id[0] );
        } elseif ( $id[1] ) {
            $author = get_the_author_meta( 'ID' );
        }

        if ( ! $author ) {
            $this->debug_info( 'it is neither post or page, applying usual rules' );
        } elseif ( user_can( $author, 'manage_options' ) && ( stripos( $matches[0], 'rel="follow"' ) !== false || stripos( $matches[0], "rel='follow'" ) !== false ) ) {
            $this->debug_info( 'This link has a follow atribute and is posted by admin, not masking it.' );

            // wordpress adds rel="nofollow" by itself when posting new link in comments. get rid of it! Also, remove our follow attibute - it is unneccesary.
            return str_ireplace( array( 'rel="follow"', "rel='follow'", 'rel="nofollow"' ), '', $matches[0] );
        } else {
            $this->debug_info( 'it does not have rel follow or is not posted by admin, masking it' );
        }

        return false;

    }

    /**
     * Checks link against the exclusion list.
     *
     * @since     4.0.0
     * @param     array    $matches
     * @return    bool
     */
    public function check_excl_list( $matches ) {

        $check_allowed = $matches[2];

        $this->debug_info( 'Checking link "' . $check_allowed . '" VS exclusion list {' . var_export( $this->exclusion_list, 1 ) . '}' );

        foreach ( $this->exclusion_list as $val ) {
            if ( stripos( $check_allowed, $val ) === 0 ) {
                $this->debug_info( 'In exclusion list (' . $val . '), not masking...' );
                return $matches[0];
            }
        }
		
		// Warith code avoid conflict with Simple Share Buttons Adder addon ask it not encrypt social sharing links  	  
		if (false !== strpos($matches[2] . '//' . $matches[3],'javascript') or false !== strpos($matches[2] . '//' . $matches[3],'mailto') or false !== strpos($matches[2] . '//' . $matches[3],'twitt') or false !== strpos($matches[2] .  '//' . $matches[3],'digi77') or false !== strpos($matches[2] . '//' . $matches[3],'skype') or false !== strpos($matches[2] . '//' . $matches[3],'#')) {
			return $matches[0];
		}
		
        $this->debug_info( 'Not in exclusion list, masking...' );

        return false;

    }

    /**
     * Checks if current page is a redirect page.
     *
     * @since    4.0.0
     */
    public function check_redirect() {

        $goto = '';
        $p = strpos( $_SERVER[ 'REQUEST_URI' ], '/' . $this->options->separator . '/' );

        if ( @$_REQUEST[ $this->options->separator ] ) {
            $goto = $_REQUEST[ $this->options->separator ];
        } elseif ( $p !== FALSE ) {
            $goto = substr( $_SERVER[ 'REQUEST_URI' ], $p + strlen( $this->options->separator ) + 2 );
        }

        $goto = strip_tags( $goto );

        if ( $goto ) {
            $this->redirect( $goto );
        }

    }

    /**
     * Initiates the redirect.
     *
     * @since    4.0.0
     * @param    string    $url
     */
    public function redirect( $url ) {

        global $wp_rewrite, $hyper_cache_stop;

        // disable Hyper Cache plugin (http://www.satollo.net/plugins/hyper-cache) from caching this page
        $hyper_cache_stop = true;

        // disable WP Super Cache caching
        if ( ! defined( 'DONOTCACHEPAGE' ) ) {
            define( 'DONOTCACHEPAGE', 1 );
        }

        $url = $this->decode_link( $url );

        $this->add_log( $url );

        if ( ! $wp_rewrite->using_permalinks() ) {
            $url = urldecode( $url );
        }

        $url = str_ireplace( '&#038;', '&', $url );

        // checking for spammer attack, redirect should happen from your own website
        if ( $this->options->check_referrer ) {
            if ( stripos( wp_get_referer(), $this->site ) !== 0 ) {
                $this->show_referrer_warning();
            }
        }

        $this->show_redirect_page( $url );

    }

    /**
     * Encodes url.
     *
     * @since     4.0.0
     * @param     string    $url
     * @return    mixed
     */
    public function encode_link( $url ) {

        global $wpdb;

        if ( 'base64' === $this->options->link_encoding ) {
            // Warith code encrypt the URL with the given salt key 
			$salt ='soprH8m30wendhKJDMQAXOEUGPD821';
			$url=trim(base64_encode(mcrypt_encrypt(MCRYPT_RIJNDAEL_256, $salt, $url, MCRYPT_MODE_ECB, mcrypt_create_iv(mcrypt_get_iv_size(MCRYPT_RIJNDAEL_256, MCRYPT_MODE_ECB), MCRYPT_RAND))));
			//$url = base64_encode( $url );
        } elseif ( 'numbers' === $this->options->link_encoding ) {

            $table_name = $wpdb->prefix . 'external_links_masks';
            $sql = 'SELECT id FROM ' . $table_name . ' WHERE LOWER(url) = LOWER(%s) LIMIT 1';
            $result = $wpdb->get_var( $wpdb->prepare( $sql, $url ) );

            // no table found
            if ( is_null( $result ) && strpos( $wpdb->last_error, "doesn't exist" ) ) {

                $charset_collate = $wpdb->get_charset_collate();

                $sql = "CREATE TABLE $table_name (
                   id bigint(20) UNSIGNED NOT NULL AUTO_INCREMENT,
                   url varchar(255) DEFAULT '' NOT NULL,
                   PRIMARY KEY  (id),
                   UNIQUE KEY  (url)
               ) $charset_collate;";

                require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
                $create = dbDelta( $sql );

                if ( empty( $create ) ) {
                    $this->debug_info(
                        __( 'Failed SQL: ', $this->plugin_name ) . '<br>' .
                        $sql . '<br>' .
                        __( 'Unable to create table.', $this->plugin_name )
                    );
                } else {
                    $wpdb->insert(
                        $wpdb->prefix . 'external_links_masks',
                        array( 'url' => $url )
                    );
                }

            } elseif ( is_null( $result ) ) {
                $this->debug_info(
                    __( 'Failed SQL: ', $this->plugin_name ) . '<br>' .
                    $sql . '<br>' .
                    __( 'Error was:', $this->plugin_name ) . '<br>' .
                    $wpdb->last_error
                );
            }

            if ( ! $result ) {
                $insert = $wpdb->insert(
                    $wpdb->prefix . 'external_links_masks',
                    array( 'url' => $url )
                );

                if ( 0 === $insert ) {
                    $this->debug_info(
                        __( 'Failed SQL: ', $this->plugin_name ) . '<br>' .
                        $sql . '<br>' .
                        __( 'Error was:', $this->plugin_name ) . '<br>' .
                        $wpdb->last_error
                    );
                }

                $url = $wpdb->insert_id;
            } else {
                $url = $result;
            }
        }

        return $url;

    }

    /**
     * Decodes encoded url.
     *
     * @since     4.0.0
     * @param     string    $url
     * @return    mixed     $url
     */
    public function decode_link( $url ) {

        global $wpdb;

        if ( 'base64' === $this->options->link_encoding ) {
            // Warith code decrypt the URL with the given salt key 
			$salt ='soprH8m30wendhKJDMQAXOEUGPD821';
	        $url=trim(mcrypt_decrypt(MCRYPT_RIJNDAEL_256, $salt, base64_decode($url), MCRYPT_MODE_ECB, mcrypt_create_iv(mcrypt_get_iv_size(MCRYPT_RIJNDAEL_256, MCRYPT_MODE_ECB), MCRYPT_RAND)));
			//$url = base64_decode( $url );
        } elseif ( 'numbers' === $this->options->link_encoding ) {
            $table_name = $wpdb->prefix . 'external_links_masks';
            $sql = 'SELECT url FROM ' . $table_name . ' WHERE id = %s LIMIT 1';
            $url = $wpdb->get_var( $wpdb->prepare( $sql, $url ) );
        }

        return $url;

    }

    /**
     * Adds log record to database.
     *
     * @since    4.0.0
     * @param    string    $url
     */
    public function add_log( $url ) {

        global $wpdb;

        if ( ! $this->options->logging ) {
            return;
        }

        $insert = $wpdb->insert(
            $wpdb->prefix . 'external_links_logs',
            array(
                'url' => $url,
                'date' => current_time( 'mysql' )
            )
        );

        if ( false !== $insert ) {
            return; // all ok
        }

        // error - stats record could not be created
        $this->debug_info(
            __( 'Failed SQL: ', $this->plugin_name ) . '<br>' .
            $wpdb->last_query . '<br>' .
            __( 'Error was:', $this->plugin_name ) . '<br>' .
            $wpdb->last_error
        );

        $charset_collate = $wpdb->get_charset_collate();
        $table_name = $wpdb->prefix . 'external_links_logs';

        $sql = "CREATE TABLE $table_name (
                   id bigint(20) UNSIGNED NOT NULL AUTO_INCREMENT,
                   url varchar(255) DEFAULT '' NOT NULL,
                   date datetime DEFAULT '0000-00-00 00:00:00' NOT NULL,
                   PRIMARY KEY  (id)
               ) $charset_collate;";

        require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
        $create = dbDelta( $sql );

        if ( empty( $create ) ) {
            $this->debug_info(
                __( 'Failed SQL: ', $this->plugin_name ) . '<br>' .
                $sql . '<br>' .
                __( 'Unable to create table.', $this->plugin_name )
            );
        } else {
            $wpdb->insert(
                $wpdb->prefix . 'external_links_logs',
                array(
                    'url' => $url,
                    'date' => current_time( 'mysql' )
                )
            );
        }

    }

    /**
     * Renders the referrer warning page.
     */
    public function show_referrer_warning() {

        include_once 'partials/referrer-warning.php';

    }

    /**
     * Renders the redirect page.
     *
     * @since    4.0.0
     */
    public function show_redirect_page( $url ) {

        header( 'Content-type: text/html; charset="utf-8"', true );

        if ( '302' === $this->options->masking_type && $url ) {
            @header( 'Location: ' . $url );
        }

        include_once 'partials/redirect.php';

    }

    /**
     * Outputs the debug log.
     *
     * TODO: Look to move to admin class
     *
     * @since    4.0.0
     */
    public function output_debug() {

        echo "\n<!--wp-noexternallinks debug:\n" . implode( "\n\n", $this->debug_log ) . "\n-->";

    }

    /**
     * Adds data to the debug log.
     *
     * TODO: Look to move to admin class
     *
     * @since      4.0.0
     * @param      string    $info
     * @param      int       $return
     * @return     string
     */
    public function debug_info( $info, $return = 0 ) {

        if ( $this->options->debug_mode ) {
            $t = "\n<!--wp-noexternallinks debug:\n" . $info . "\n-->";
            $this->debug_log[] = $info;

            if ( $return ) {
                return $t;
            }
        }

        return '';

    }

    /**
     * Generates the exclusion list.
     *
     * @since     4.0.0
     * @return    array    $exclude_links
     */
    public function get_exclusion_list() {

        $site = str_replace( array( 'http://', 'https://' ), '', $this->site );

        $exclude_links = array();
        $exclude_links[] = "http://" . $site;
        $exclude_links[] = "https://" . $site;
        $exclude_links[] = 'javascript';
        $exclude_links[] = 'mailto';
        $exclude_links[] = 'skype';
        $exclude_links[] = '/'; // for relative links
        $exclude_links[] = '#'; // for internal links

        foreach ( explode( "\r\n", $this->options->exclusion_list ) as $item ) {
            $exclude_links[] = $item;
        }

        return $exclude_links;

    }

}
